﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Diagnostics;
using System.IO;

namespace NanoTaskLogger {
	public partial class Form1 : Form {
		static private string prevProcName    = "";
		static private string prevFilePath    = "";
		static private string prevProductName = "";

		[DllImport("user32.dll")]
		public static extern IntPtr GetForegroundWindow();

		[DllImport("user32.dll")]
		public static extern int GetWindowThreadProcessId(IntPtr hWnd, out int lpdwProcessId);

		[DllImport("user32.dll")]
		private static extern int GetWindowText(IntPtr hWnd, StringBuilder text, int length);


		public Form1() {
			InitializeComponent();
		}

		//*********************************************************************
		// 一定周期でログを取る
		//*********************************************************************
		private void timerInterval_Tick( object sender, EventArgs e ) {
			// 現在、最前面ウィンドウのプロセスオブジェクトを取得する
			Process proc = Process.GetCurrentProcess();

			string procName;
			string productName;
			string filePath;
			try { 
				procName =  GetActiveWindowTitle();
			} catch {
				procName = "unknown";
			}

			try { 
				filePath = proc.MainModule.FileName + Environment.NewLine;
			} catch {
				filePath = "unknown" + Environment.NewLine;
			}

			// ファイルのプロダクト情報(アプリによって登録状況が異なるので使用しない)
			try { 
				productName = proc.MainModule.FileVersionInfo.ProductName + Environment.NewLine;;
			} catch {
				productName = "unknown" + Environment.NewLine;
			}

			// アクティブなプロセスが変わってなければ何もしない
			if ( procName.Equals( prevProcName ) && 
				 filePath.Equals( prevFilePath ) && 
				 productName.Equals( prevProductName ) ) {
				return;
			}

			// 一件前の情報を覚える
			prevProcName = procName;
			prevFilePath = filePath;
			prevProductName = productName;

			// ログメッセージを組み立てる
			string logMessage = DateTime.Now.ToString() + "\t" + 
								productName.Replace( "\t", " "  ).Replace( Environment.NewLine, "" ) + "\t" + 
								procName.Replace( "\t", " " ).Replace( Environment.NewLine, "" ) + "\t" + 
								filePath;

			WriteLog( logMessage );
			textLog.Text += logMessage;
		}

		//*********************************************************************
		/// <summary> ログをファイルに出力する
		/// </summary>
		/// <param name="logMessage">出力するメッセージ</param>
		//*********************************************************************
		public static void WriteLog( string logMessage ) {
			try {
				string fileName = "log.txt";


				// ファイルにログを出力
				using ( FileStream   stream = new FileStream( fileName, FileMode.Append, FileAccess.Write, FileShare.ReadWrite ) )
				using ( StreamWriter writer = new StreamWriter( stream, Encoding.GetEncoding(932) ) ) {	
					writer.WriteLine( logMessage );
				}
			} catch {
				// 所詮ログファイルなので、書き込みに失敗しても無視する
			}
		}

		//*********************************************************************
		/// <summary> 現在最前面に表示されているウィンドウのProcessオブジェクトを取得
		///				see also: http://mzs184.blogspot.jp/2009/03/c.html
		/// </summary>
		/// <returns></returns>
		//*********************************************************************
		public static Process GetActiveProcess()
		{
			int id;
			// アクティブなウィンドウハンドルの取得
			IntPtr hWnd = GetForegroundWindow();

			// ウィンドウハンドルからプロセスIDを取得
			GetWindowThreadProcessId( hWnd, out id );
	
			Process process = Process.GetProcessById(id);
			return process;
		}

		//*********************************************************************
		/// <summary> アクティブなウィンドウのタイトルを取得
		///			see also: http://d.hatena.ne.jp/int128/20080110/1199975050
		/// </summary>
		/// <returns></returns>
		//*********************************************************************
		public static string GetActiveWindowTitle()
		{
			IntPtr hWnd = GetForegroundWindow();
			StringBuilder title = new StringBuilder(1048);

			// ウィンドウタイトルを取得
			GetWindowText(hWnd, title, 1024);
			return title.ToString();
		}
	}
}